@@ -5,7 +5,11 @@ import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
 
+# This is for filtering cosmic rays
+from DiadFit.cosmicray_filter import *
 
+# This is for processing calibration pressures with the FDCA
+from DiadFit.Psensor import *
 
 # This reads different data formats, imports files, and does the string stripping for WITEC
 from DiadFit.importing_data_files import *
@@ -21,7 +25,6 @@ from DiadFit.diads import *
 # This has densimeters from different instruments
 from DiadFit.densimeters import *
 
-
 # Skewness
 from DiadFit.skewness import *
 
@@ -32,6 +35,7 @@ from DiadFit.H2O_fitting import *
 from DiadFit.error_propagation import *
 
 from DiadFit.density_depth_crustal_profiles import *
+from DiadFit.CO2_EOS import *
 
 # version
 from ._version import __version__
