@@ -3,7 +3,7 @@ import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib import patches
 import lmfit
-from lmfit.models import GaussianModel, VoigtModel, LinearModel, ConstantModel
+from lmfit.models import GaussianModel, VoigtModel, LinearModel, ConstantModel, PseudoVoigtModel
 from scipy.signal import find_peaks
 import os
 import re
@@ -13,9 +13,10 @@ from DiadFit.importing_data_files import *
 from typing import Tuple, Optional
 from dataclasses import dataclass
 import matplotlib.patches as patches
-from tqdm import tqdm
-
+from tqdm.notebook import tqdm
+import re
 
+## Test this
 
 
 encode="ISO-8859-1"
@@ -133,9 +134,32 @@ def calculate_Ne_line_positions(wavelength=532.05, cut_off_intensity=2000):
     df_Ne_r=df_Ne.loc[df_Ne['Intensity']>cut_off_intensity]
     return df_Ne_r
 
-def plot_Ne_lines(*, path=None, filename=None, filetype=None, n_peaks=6,
-peak1_cent=1118, peak2_cent=1447, exclude_range_1=None, Ne_array=None,
-exclude_range_2=None, height=10, threshold=0.6, distance=1, prominence=10, width=1,):
+
+@dataclass
+class Neon_id_config:
+    # Exclude a range, e.g. cosmic rays
+    exclude_range_1: Optional [Tuple[float, float]] = None
+    exclude_range_2: Optional [Tuple[float, float]] = None
+
+    # Things for Scipy find peaks
+    height: tuple  = 10
+    distance: float= 1
+    prominence: float = 10
+    width: float = 1
+    threshold: float = 0.6
+    peak1_cent: float= 1117
+    peak2_cent: float=1447
+
+    # Number of peaks to look for
+    n_peaks: float = 6
+
+
+
+
+
+
+def identify_Ne_lines(*, config: Neon_id_config=Neon_id_config(), path=None, filename=None, filetype=None, plot_figure=True, print_df=False,
+Ne_array=None):
 
     """
     Loads Ne line, uses scipy find peaks to identify peaks, overlays these,
@@ -182,14 +206,27 @@ exclude_range_2=None, height=10, threshold=0.6, distance=1, prominence=10, width
 
     """
 
+    # This bit extracts the data, unless you already fed it in as an array
     if filename is not None and path is not None and filetype is not None:
         Ne_in=get_data(path=path, filename=filename, filetype=filetype)
     if Ne_array is not None:
         Ne_in=Ne_array
 
 
+    # This gets parameters from config file
+    exclude_range_1=config.exclude_range_1
+    exclude_range_2=config.exclude_range_2
+    height=config.height
+    distance=config.distance
+    prominence=config.prominence
+    width=config.width
+    threshold=config.threshold
+    peak1_cent=config.peak1_cent
+    peak2_cent=config.peak2_cent
+    n_peaks=config.n_peaks
 
-    # Exclude range
+
+    # This bit filters the spectra if you want to exclude a range or 2
     if exclude_range_1 is None and exclude_range_2 is None:
         Ne=Ne_in
     if exclude_range_1 is not None:
@@ -198,106 +235,149 @@ exclude_range_2=None, height=10, threshold=0.6, distance=1, prominence=10, width
     if exclude_range_2 is not None:
         Ne=Ne_in[ (Ne_in[:, 0]<exclude_range_2[0]) | (Ne_in[:, 0]>exclude_range_2[1]) ]
 
-    # Find peaks
+    # Get X and Y coords.
     y=Ne[:, 1]
     x=Ne[:, 0]
-
-    fig, (ax0, ax1, ax2) = plt.subplots(1, 3, figsize=(9, 3), sharey=True)
-    ax0.plot(x, y, '-r')
-    miny=np.min(y)
-    maxy=np.max(y)
-    ax0.plot([1117, 1117], [miny, maxy], ':k')
-    ax0.plot([1220, 1220], [miny, maxy], ':k')
-    ax0.plot([1310, 1310], [miny, maxy], ':k')
-    ax0.plot([1398, 1398], [miny, maxy], ':k')
-    ax0.plot([1447, 1447], [miny, maxy], ':k')
-    ax0.plot([1566, 1566], [miny, maxy], ':k')
-
-    ax0.set_ylabel('Amplitude (counts)')
-    ax0.set_xlabel('Wavenumber (cm$^{-1}$)')
+    spec_res=np.abs(x[1]-x[0])
 
 
+    # Apply Scipy Find peaks using the parameters in the config file.
     peaks = find_peaks(y,height = height, threshold = threshold, distance = distance, prominence=prominence, width=width)
-    # print('Found peaks at wavenumber=')
-    # print(x[peaks[0]])
 
-    n_peaks=6
-    height = peaks[1]['peak_heights'] #list of the heights of the peaks
-    peak_pos = x[peaks[0]] #list of the peaks positions
+    # This gets a list of peak heights
+    height = peaks[1]['peak_heights']
+    # This gets a list of peak positions
+    peak_pos = x[peaks[0]]
+
+    # Lets combine them in a dataframe
     df=pd.DataFrame(data={'pos': peak_pos,
                         'height': height})
 
 
-    # Find bigest peaks,
+    # Find bigest peaks, and take up to n peaks
     df_sort_Ne=df.sort_values('height', axis=0, ascending=False)
     df_sort_Ne_trim=df_sort_Ne[0:n_peaks]
 
-    print('Biggest 6 peaks:')
-    display(df_sort_Ne_trim)
+    if print_df is True:
+        print('Biggest 6 peaks:')
+        display(df_sort_Ne_trim)
 
-    df_pk1=df_sort_Ne.loc[df['pos'].between(peak1_cent-5, peak1_cent+5)]
-    df_pk2=df_sort_Ne.loc[df['pos'].between(peak2_cent-5, peak2_cent+5)]
+    # Get peak within +-5
+    df_pk1=df_sort_Ne.loc[df['pos'].between(peak1_cent-10*spec_res, peak1_cent+10*spec_res)]
+    df_pk2=df_sort_Ne.loc[df['pos'].between(peak2_cent-10*spec_res, peak2_cent+10*spec_res)]
 
     df_pk1_trim=df_pk1[0:1]
     df_pk2_trim=df_pk2[0:1]
 
+    # Lets extract spectra 10 spectral units either side of the asked for peak positions
+    Neon1_region=(x<(peak1_cent+10*spec_res)) & (x>(peak1_cent-10*spec_res))
+    Neon1_trim_y=y[Neon1_region]
+    Neon1_trim_x=x[Neon1_region]
+    Neon2_region=(x<(peak1_cent+10*spec_res)) & (x>(peak1_cent-10*spec_res))
+    Neon2_trim_y=y[Neon1_region]
+    Neon2_trim_x=x[Neon1_region]
+
+    # Take 25th quantile as representative of the background position
+    Baseline_Neon1=np.quantile(Neon1_trim_y, 0.1)
+    Baseline_Neon2=np.quantile(Neon2_trim_y, 0.1)
+
+    if len(df_pk1_trim)==1:
+
+        df_fit_params=pd.DataFrame(data={'Peak1_cent': df_pk1_trim['pos'],
+                                        'Peak1_height': df_pk1_trim['height']})
+    elif len(df_pk1_trim)>1:
+        df_pk1_max=df_pk1_trim.loc[df_pk1_trim['height']==np.max(df_pk1_trim['height'])]
+        df_fit_params=pd.DataFrame(data={'Peak1_cent': df_pk1_max['pos'],
+                                        'Peak1_height': df_pk1_max['height']})
+    elif len(df_pk1_trim)==0:
 
 
-    ax1.plot(Ne_in[:, 0], Ne_in[:, 1], '-c', label='input')
-    ax1.plot(x, y, '-r', label='filtered')
-    ax1.plot(df_sort_Ne_trim['pos'], df_sort_Ne_trim['height'], '*c', label='all peaks')
+        Max_y_Neon1=np.max(Neon1_trim_y)
+        x_Neon_1=x[y==Max_y_Neon1][0]
+        df_fit_params=pd.DataFrame(data={'Peak1_cent': x_Neon_1,
+                                    'Peak1_height': Max_y_Neon1}, index=[0])
 
-    if len(df_pk1_trim)==0:
-        print('No peak found within +-5 wavenumbers of peak position 1, have returned user-entered peak')
+
+    if len(df_pk2_trim)==1:
+        df_fit_params['Peak2_cent']=df_pk2_trim['pos'].iloc[0]
+        df_fit_params['Peak2_height']=df_pk2_trim['height'].iloc[0]
+    elif len(df_pk2_trim)>1:
+        df_pk2_max=df_pk2_trim.loc[df_pk2_trim['height']==np.max(df_pk2['height'])]
+
+        df_fit_params['Peak2_cent']=df_pk2_max['pos'].iloc[0]
+        df_fit_params['Peak2_height']=df_pk2_max['height'].iloc[0]
+
+    elif len(df_pk2_trim)==0:
+
+        Max_y_Neon2=np.max(Neon2_trim_y)
+        x_Neon_2=x[y==Max_y_Neon2][0]
+        df_fit_params['Peak2_cent']= x_Neon_2
+        df_fit_params['Peak2_height']=Max_y_Neon2
+
+
+    if plot_figure is True:
+        fig, (ax0, ax1, ax2) = plt.subplots(1, 3, figsize=(11, 3.5))
+        ax0.plot(x, y, '-r')
+        miny=np.min(y)
+        maxy=np.max(y)
+
+
+        ax0.set_ylabel('Amplitude (counts)')
+        ax0.set_xlabel('Wavenumber (cm$^{-1}$)')
+        ax1.plot(Ne_in[:, 0], Ne_in[:, 1], '-c')
+        ax1.plot(Ne_in[:, 0], Ne_in[:, 1], '.c')
+        ax1.plot(Ne[:, 0], Ne[:, 1], '.r')
+        ax1.plot(x, y, '-r', label='filtered')
+        ax1.plot(df_sort_Ne_trim['pos'], df_sort_Ne_trim['height'], '*c', label='all pks IDed')
+
+        ax1.plot([peak1_cent-20, peak1_cent+20], [Baseline_Neon1, Baseline_Neon1], '-g', label='approx bck' )
         #ax1.plot(peak2_cent, df_pk2_trim['height'], '*k')
-        pos_pk1=str(peak1_cent)
-        ax1.annotate(pos_pk1, xy=(peak1_cent,
-        100-10), xycoords="data", fontsize=10, rotation=90)
+
+        pos_pk1=str(np.round(df_fit_params['Peak1_cent'].iloc[0], 2))
+
         nearest_pk1=peak1_cent
 
-    else:
-        ax1.plot(df_pk1_trim['pos'], df_pk1_trim['height'], '*k', mfc='yellow', ms=8, label='selected peak')
-        pos_pk1=str(np.round(df_pk1_trim['pos'].iloc[0], 1))
-        ax1.annotate(pos_pk1, xy=(df_pk1_trim['pos']-5,
-        df_pk1_trim['height']-10), xycoords="data", fontsize=10, rotation=90)
-        nearest_pk1=float(df_pk1_trim['pos'])
 
-    if len(df_pk2_trim)==0:
-        print('No peak found within +-5 wavenumbers of peak position 2, have returned user-entered peak')
-        pos_pk2=str(peak2_cent)
-        nearest_pk2=peak2_cent
-        ax2.annotate(pos_pk2, xy=(nearest_pk2,
-        200), xycoords="data", fontsize=10, rotation=90)
-
-    else:
-        ax2.plot(df_pk2_trim['pos'], df_pk2_trim['height'], '*k', mfc='yellow', ms=8,label='selected peak')
-        ax2.legend(bbox_to_anchor=(0.3,1.1))
-
-        pos_pk2=str(np.round(df_pk2_trim['pos'].iloc[0], 1))
-        nearest_pk2=float(df_pk2_trim['pos'])
-
-        ax2.annotate(pos_pk2, xy=(df_pk2_trim['pos']-5,
-        df_pk2_trim['height']/2), xycoords="data", fontsize=10, rotation=90)
-
-    ax1.set_xlim([peak1_cent-15, peak1_cent+15])
-
-    ax1.set_xlim([peak1_cent-10, peak1_cent+10])
-
-    ax2.plot(x, y, '-r')
-    ax2.plot(df_sort_Ne_trim['pos'], df_sort_Ne_trim['height'], '*k', mfc='yellow', ms=8)
-    #print(df_pk1)
+        ax1.annotate('peak=' + pos_pk1, xy=(df_fit_params['Peak1_cent'].iloc[0]+3,
+        Baseline_Neon1*1.5+700), xycoords="data", fontsize=10, rotation=90)
 
 
-    ax2.set_xlim([peak2_cent-15, peak2_cent+15])
+        ax1.plot(df_fit_params['Peak1_cent'], df_fit_params['Peak1_height'], '*k', mfc='yellow', ms=8, label='selected peak')
 
-    ax1.set_xlabel('Wavenumber (cm$^{-1}$)')
-    ax2.set_xlabel('Wavenumber (cm$^{-1}$)')
+        ax1.legend(loc='upper center', ncol=2, fontsize=8)
 
-    print('selected Peak 1 Pos')
-    print(nearest_pk1)
-    print('selected Peak 2 Pos')
-    print(nearest_pk2)
-    return Ne, df_sort_Ne_trim, nearest_pk1, nearest_pk2
+
+        pos_pk2=str(np.round(df_fit_params['Peak2_cent'].iloc[0], 2))
+        ax2.plot([peak2_cent-20, peak2_cent+20], [Baseline_Neon2, Baseline_Neon2], '-g')
+
+        ax2.annotate('peak=' + pos_pk2, xy=(df_fit_params['Peak2_cent'].iloc[0]-5,
+        Baseline_Neon1*2+700), xycoords="data", fontsize=10, rotation=90)
+
+
+
+        ax1.set_xlim([peak1_cent-15, peak1_cent+15])
+
+        ax1.set_xlim([peak1_cent-10, peak1_cent+10])
+        ax2.plot(Ne[:, 0], Ne[:, 1], '.r', label='input')
+        ax2.plot(x, y, '-r')
+        ax2.plot(df_sort_Ne_trim['pos'], df_sort_Ne_trim['height'], '*k', mfc='yellow', ms=8)
+        #print(df_pk1)
+
+
+        ax2.set_xlim([peak2_cent-15, peak2_cent+15])
+
+        ax1.set_xlabel('Wavenumber (cm$^{-1}$)')
+        ax2.set_xlabel('Wavenumber (cm$^{-1}$)')
+
+        ax1.set_ylim([0, 1.5*df_fit_params['Peak1_height'].iloc[0]+100])
+        ax2.set_ylim([0, 1.5*df_fit_params['Peak2_height'].iloc[0]+100])
+        fig.tight_layout()
+
+    df_fit_params['Peak1_prom']=df_fit_params['Peak1_height']-Baseline_Neon1
+    df_fit_params['Peak2_prom']=df_fit_params['Peak2_height']-Baseline_Neon2
+
+
+    return Ne, df_fit_params
 
 
 
@@ -449,8 +529,9 @@ lower_bck=None, upper_bck1=None, upper_bck2=None):
 
 
 
-def fit_pk1(x, y_corr, x_span=[-10, 8], Ne_center=1117.1, amplitude=98, sigma=0.28,
-LH_offset_mini=[1.5, 3], peaks_pk1=2, block_print=True) :
+def fit_pk1(x, y_corr, x_span=[-10, 8], Ne_center=1117.1, amplitude=98, pk1_sigma=0.28,
+LH_offset_mini=[1.5, 3], peaks_pk1=2, model_name='PseudoVoigtModel', block_print=True,
+const_params=True, spec_res=0.4) :
     """ This function fits the 1117 Ne line as 1 or two voigt peaks
 
     Parameters
@@ -486,6 +567,12 @@ LH_offset_mini=[1.5, 3], peaks_pk1=2, block_print=True) :
 
 
     """
+    if const_params is True:
+        min_off=0.8
+        max_off=1.2
+    if const_params is False:
+        min_off=0
+        max_off=100
 
     # Flatten x and y if needed
     xdat=x.flatten()
@@ -504,7 +591,10 @@ LH_offset_mini=[1.5, 3], peaks_pk1=2, block_print=True) :
     if peaks_pk1>1:
 
         # Setting up lmfit
-        model0 = VoigtModel(prefix='p0_')#+ ConstantModel(prefix='c0')
+        if model_name == 'PseudoVoigtModel':
+            model0 = PseudoVoigtModel(prefix='p0_')#+ ConstantModel(prefix='c0')
+        if model_name=="VoigtModel":
+            model0 = VoigtModel(prefix='p0_')#+ ConstantModel(prefix='c0')
         pars0 = model0.make_params(p0_center=Ne_center, p0_amplitude=amplitude)
         init0 = model0.eval(pars0, x=xdat)
         result0 = model0.fit(ydat, pars0, x=xdat)
@@ -517,21 +607,37 @@ LH_offset_mini=[1.5, 3], peaks_pk1=2, block_print=True) :
         if block_print is False:
             print('first iteration, peak Amplitude='+str(np.round(Amp_p0, 4)))
         fwhm_p0=result0.params.get('p0_fwhm')
-        Center_p0_errorval=float(str(Center_p0_error).split()[4].replace(",", ""))
+
+
+        pattern = r"\+/-\s*([\d.]+)"
+        match = re.search(pattern, str(Center_p0_error))
+        if match:
+            Center_p0_errorval = float(match.group(1))
+        else:
+            Center_p0_errorval=np.nan
+
+
 
         #Ne_center=Ne_center
         #rough_peak_positions=Ne_center-2
-
-        model1 = VoigtModel(prefix='p1_')#+ ConstantModel(prefix='c1')
+        if model_name == 'PseudoVoigtModel':
+            model1 = PseudoVoigtModel(prefix='p1_')#+ ConstantModel(prefix='c0')
+        if model_name=="VoigtModel":
+            model1 = VoigtModel(prefix='p1_')#+ ConstantModel(prefix='c0')
         pars1 = model1.make_params()
-        pars1['p1_'+ 'amplitude'].set(Amp_p0, min=0.8*Amp_p0, max=1.2*Amp_p0)
+        pars1['p1_'+ 'amplitude'].set(Amp_p0, min=min_off*Amp_p0, max=max_off*Amp_p0)
         pars1['p1_'+ 'center'].set(Center_p0, min=Center_p0-0.2, max=Center_p0+0.2)
-        pars1['p1_'+ 'fwhm'].set(fwhm_p0, min=fwhm_p0-1, max=fwhm_p0+1)
+        pars1['p1_'+ 'sigma'].set(pk1_sigma, min=pk1_sigma*min_off, max=pk1_sigma*max_off)
 
 
         # Second wee peak
         prefix='p2_'
-        peak = VoigtModel(prefix='p2_')#+ ConstantModel(prefix='c2')
+        if model_name == 'PseudoVoigtModel':
+            peak = PseudoVoigtModel(prefix='p2_')#+ ConstantModel(prefix='c0')
+        if model_name=="VoigtModel":
+            peak = VoigtModel(prefix='p2_')#+ ConstantModel(prefix='c0')
+
+
         pars = peak.make_params()
         minp2=Center_p0-LH_offset_mini[1]
         maxp2=Center_p0-LH_offset_mini[0]
@@ -551,16 +657,19 @@ LH_offset_mini=[1.5, 3], peaks_pk1=2, block_print=True) :
 
 
     if peaks_pk1==1:
-        if block_print is False:
-            print('fitting a single peak, if you want the shoulder, do peaks_pk1=2')
 
-        model_combo = VoigtModel(prefix='p1_')#+ ConstantModel()
+        if model_name == 'PseudoVoigtModel':
+            model_combo = PseudoVoigtModel(prefix='p1_')#+ ConstantModel(prefix='c0')
+        if model_name=="VoigtModel":
+            model_combo= VoigtModel(prefix='p1_')#+ ConstantModel(prefix='c0')
+
 
 
         # create parameters with initial values
-        pars1 = model_combo.make_params(amplitude=amplitude, sigma=sigma)
-        pars1['p1_' + 'center'].set(Ne_center, min=Ne_center-2,
-        max=Ne_center+2)
+        pars1 = model_combo.make_params(amplitude=amplitude)
+        pars1['p1_' + 'center'].set(Ne_center, min=Ne_center-2*spec_res,max=Ne_center+2*spec_res)
+        pars1['p1_'+ 'sigma'].set(pk1_sigma, min=pk1_sigma*min_off, max=pk1_sigma*max_off)
+
 
 
 
@@ -577,6 +686,9 @@ LH_offset_mini=[1.5, 3], peaks_pk1=2, block_print=True) :
     Center_p1=result.best_values.get('p1_center')
     Center_p1_error=result.params.get('p1_center')
 
+    # Get mix of lorenz
+    Peak1_Prop_Lor=result.best_values.get('p1_fraction')
+
 
     if peaks_pk1==1:
         Center_pk1=Center_p1
@@ -644,10 +756,11 @@ LH_offset_mini=[1.5, 3], peaks_pk1=2, block_print=True) :
 
 
 
-    return Center_pk1, Area_pk1, sigma_pk1, gamma_pk1, Ne_pk1_reg_x_plot, Ne_pk1_reg_y_plot, Ne_pk1_reg_x, Ne_pk1_reg_y, xx_pk1, result_pk1, error_pk1, result_pk1_origx, comps
+    return Center_pk1, Area_pk1, sigma_pk1, gamma_pk1, Ne_pk1_reg_x_plot, Ne_pk1_reg_y_plot, Ne_pk1_reg_x, Ne_pk1_reg_y, xx_pk1, result_pk1, error_pk1, result_pk1_origx, comps, Peak1_Prop_Lor
 
 
-def fit_pk2(x, y_corr, x_span=[-5, 5], Ne_center=1447.5, amplitude=1000, sigma=0.28, print_report=False) :
+def fit_pk2(x, y_corr, x_span=[-5, 5], Ne_center=1447.5, amplitude=1000, pk2_sigma=0.4,
+model_name='PseudoVoigtModel', print_report=False, const_params=True) :
     """ This function fits the 1447 Ne line as a single Voigt
 
     Parameters
@@ -678,6 +791,13 @@ def fit_pk2(x, y_corr, x_span=[-5, 5], Ne_center=1447.5, amplitude=1000, sigma=0
 
 
     """
+    if const_params is True:
+        min_off=0.8
+        max_off=1.2
+    if const_params is False:
+        min_off=0
+        max_off=100
+
 
     # This defines the range you want to fit (e.g. how big the tails are)
     lower_pk2=Ne_center+x_span[0]
@@ -689,34 +809,42 @@ def fit_pk2(x, y_corr, x_span=[-5, 5], Ne_center=1447.5, amplitude=1000, sigma=0
     Ne_pk2_reg_x_plot=x[(x>(lower_pk2-3))&(x<(upper_pk2+3))]
     Ne_pk2_reg_y_plot=y_corr[(x>(lower_pk2-3))&(x<(upper_pk2+3))]
 
+    if model_name == 'PseudoVoigtModel':
+        model = PseudoVoigtModel()#+ ConstantModel(prefix='c0')
+    if model_name=="VoigtModel":
+        model = VoigtModel()#+ ConstantModel(prefix='c0')
 
-    model = VoigtModel()#+ ConstantModel()
 
 
     # create parameters with initial values
-    params = model.make_params(center=Ne_center, amplitude=amplitude, sigma=sigma)
+    params = model.make_params(center=Ne_center, amplitude=amplitude, sigma=pk2_sigma)
 
     # Place bounds on center allowed
     params['center'].min = Ne_center+x_span[0]
     params['center'].max = Ne_center+x_span[1]
-
-
-
+    params['sigma'].max = pk2_sigma*max_off
+    params['sigma'].min = pk2_sigma*min_off
+    params['amplitude'].max = amplitude*max_off
+    params['amplitude'].min = amplitude*min_off
     result = model.fit(Ne_pk2_reg_y.flatten(), params, x=Ne_pk2_reg_x.flatten())
 
     # Get center value
     Center_pk2=result.best_values.get('center')
     Center_pk2_error=result.params.get('center')
 
+    Peak2_Prop_Lor=result.best_values.get('fraction')
+
     #print(result.best_values)
 
     Area_pk2=result.best_values.get('amplitude')
     sigma_pk2=result.best_values.get('sigma')
     gamma_pk2=result.best_values.get('gamma')
     # Have to strip away the rest of the string, as center + error
-
-    Center_pk2_errorval=float(str(Center_pk2_error).split()[4].replace(",", ""))
-    error_pk2=Center_pk2_errorval
+    # print('debug:')
+    # print(Center_pk2_error)
+    # Center_pk2_errorval=float(str(Center_pk2_error).split()[4].replace(",", ""))
+    # error_pk2=Center_pk2_errorval
+    error_pk2=1
 
     # Evaluate the peak at 100 values for pretty plotting
     xx_pk2=np.linspace(lower_pk2, upper_pk2, 2000)
@@ -727,11 +855,13 @@ def fit_pk2(x, y_corr, x_span=[-5, 5], Ne_center=1447.5, amplitude=1000, sigma=0
     if print_report is True:
          print(result.fit_report(min_correl=0.5))
 
-    return Center_pk2,Area_pk2, sigma_pk2, gamma_pk2,  Ne_pk2_reg_x_plot, Ne_pk2_reg_y_plot, Ne_pk2_reg_x, Ne_pk2_reg_y, xx_pk2, result_pk2, error_pk2, result_pk2_origx
+    return Center_pk2,Area_pk2, sigma_pk2, gamma_pk2,  Ne_pk2_reg_x_plot, Ne_pk2_reg_y_plot, Ne_pk2_reg_x, Ne_pk2_reg_y, xx_pk2, result_pk2, error_pk2, result_pk2_origx, Peak2_Prop_Lor
 
 ## Setting default Ne fitting parameters
 @dataclass
 class Ne_peak_config:
+
+    model_name: str = 'PseudoVoigtModel'
     # Things for the background positioning and fit
     N_poly_pk1_baseline: float = 1 #Degree of polynomial to fit to the baseline
     N_poly_pk2_baseline: float = 1 #Degree of polynomial to fit to the baseline
@@ -743,13 +873,23 @@ class Ne_peak_config:
     upper_bck1_pk2: Tuple[float, float] = (15, 50) # Background position Pk2
     upper_bck2_pk2: Tuple[float, float] = (50, 51)     # Background position Pk1
 
+    # Whether you want a secondary peak
+    peaks_1: float=2
+
+    # SPlitting
+    DeltaNe_ideal: float= 330.477634
+
     # Things for plotting the baseline
-    x_range_baseline: float=20 #  How many units outside your selected background it shows on the baseline plot
-    y_range_baseline: float= 200    # Where the y axis is cut off above the minimum baseline measurement
+    x_range_baseline_pk1: float=20 #  How many units outside your selected background it shows on the baseline plot
+    y_range_baseline_pk1: float= 200    # Where the y axis is cut off above the minimum baseline measurement
+    x_range_baseline_pk2: float=20 #  How many units outside your selected background it shows on the baseline plot
+    y_range_baseline_pk2: float= 200    # Where the y axis is cut off above the minimum baseline measurement
+
+
+    # Sigma for peaks
+    pk1_sigma: float = 0.4
+    pk2_sigma: float = 0.4
 
-    # Things for fitting the primary peak
-    pk1_amplitude: float = 100
-    pk2_amplitude: float = 100
     # Things for plotting the primary peak
     x_range_peak: float=15 # How many units to each side are shown when plotting the peak fits
 
@@ -766,11 +906,11 @@ class Ne_peak_config:
 
 
 def fit_Ne_lines(*,  config: Ne_peak_config=Ne_peak_config(),
-Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
-    Ne=None, filename=None, path=None, prefix=True,
-    plot_figure=True, loop=True,
-    DeltaNe_ideal=330.477634, save_clipboard=True,
-    close_figure=False):
+Ne_center_1=1117.1, Ne_center_2=1147, Ne_prom_1=100, Ne_prom_2=200,
+Ne=None, filename=None, path=None, prefix=True,
+plot_figure=True, loop=True,
+ save_clipboard=False,
+    close_figure=False, const_params=True):
 
 
 
@@ -817,10 +957,10 @@ Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
 
     DeltaNe_ideal: float
         Theoretical distance between the two peaks you have selected. Default is 330.477634 for
-        the 1117 and 1447 diad for the Cornell Raman. You can calculate this using the calculate_Ne_line_positions
+        the 1117 and 1447 Neon for the Cornell Raman. You can calculate this using the calculate_Ne_line_positions
 
 
-    Things for Diad 1 (~1117):
+    Things for Neon 1 (~1117):
 
         N_poly_pk1_baseline: int
             Degree of polynomial used to fit the background
@@ -838,7 +978,7 @@ Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
 
 
         peaks_1: int
-            How many peaks to fit to the 1117 diad, if 2, tries to put a shoulder peak
+            How many peaks to fit to the 1117 Neon, if 2, tries to put a shoulder peak
 
         LH_offset_mini: list
             If peaks>1, puts second peak within this range left of the main peak
@@ -846,7 +986,7 @@ Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
 
 
 
-    Things for Diad 2 (~1447):
+    Things for Neon 2 (~1447):
         N_poly_pk2_baseline: int
             Degree of polynomial used to fit the background
 
@@ -862,6 +1002,16 @@ Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
             e.g. by default, fits to 10 wavenumbers below peak, 8 above.
     """
 
+    x=Ne[:, 0]
+    spec_res=np.abs(x[1]-x[0])
+    # Getting things from config file
+    peaks_1=config.peaks_1
+    DeltaNe_ideal=config.DeltaNe_ideal
+
+    # Estimate amplitude from prominence and sigma you entered
+    Pk1_Amp=((config.pk1_sigma)*(Ne_prom_1))/0.3939
+    Pk2_Amp=((config.pk2_sigma)*(Ne_prom_2))/0.3939
+
 
     #Remove the baselines
     y_corr_pk1, Py_base_pk1, x_pk1, Ne_short_pk1, Py_base_pk1, Baseline_ysub_pk1, Baseline_x_pk1=remove_Ne_baseline_pk1(Ne,
@@ -889,11 +1039,13 @@ Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
         x_span_pk2_dist=abs(config.x_span_pk2[1]-config.x_span_pk2[0])
 
     # Fit the 1117 peak
-    cent_pk1, Area_pk1, sigma_pk1, gamma_pk1, Ne_pk1_reg_x_plot, Ne_pk1_reg_y_plot, Ne_pk1_reg_x, Ne_pk1_reg_y, xx_pk1, result_pk1, error_pk1, result_pk1_origx, comps = fit_pk1(x_pk1, y_corr_pk1, x_span=x_span_pk1, Ne_center=Ne_center_1, LH_offset_mini=config.LH_offset_mini, peaks_pk1=peaks_1, amplitude=config.pk1_amplitude)
+    cent_pk1, Area_pk1, sigma_pk1, gamma_pk1, Ne_pk1_reg_x_plot, Ne_pk1_reg_y_plot, Ne_pk1_reg_x, Ne_pk1_reg_y, xx_pk1, result_pk1, error_pk1, result_pk1_origx, comps, Peak1_Prop_Lor = fit_pk1(x_pk1, y_corr_pk1, x_span=x_span_pk1, Ne_center=Ne_center_1,model_name=config.model_name,  LH_offset_mini=config.LH_offset_mini, peaks_pk1=peaks_1, amplitude=Pk1_Amp, pk1_sigma=config.pk1_sigma,
+    const_params=const_params, spec_res=spec_res)
+
 
 
     # Fit the 1447 peak
-    cent_pk2,Area_pk2, sigma_pk2, gamma_pk2, Ne_pk2_reg_x_plot, Ne_pk2_reg_y_plot, Ne_pk2_reg_x, Ne_pk2_reg_y, xx_pk2, result_pk2, error_pk2, result_pk2_origx = fit_pk2(x_pk2, y_corr_pk2, x_span=x_span_pk2,  Ne_center=Ne_center_2, amplitude=config.pk2_amplitude)
+    cent_pk2,Area_pk2, sigma_pk2, gamma_pk2, Ne_pk2_reg_x_plot, Ne_pk2_reg_y_plot, Ne_pk2_reg_x, Ne_pk2_reg_y, xx_pk2, result_pk2, error_pk2, result_pk2_origx, Peak2_Prop_Lor = fit_pk2( x_pk2, y_corr_pk2, x_span=x_span_pk2,  Ne_center=Ne_center_2, model_name=config.model_name, amplitude=Pk2_Amp, pk2_sigma=config.pk2_sigma, const_params=const_params)
 
 
     # Calculate difference between peak centers, and Delta Ne
@@ -918,18 +1070,18 @@ Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
 
         # Setting y limits of axis
         ymin_ax1=min(Ne_short_pk1[:,1])-10
-        ymax_ax1=min(Ne_short_pk1[:,1])+config.y_range_baseline
+        ymax_ax1=min(Ne_short_pk1[:,1])+config.y_range_baseline_pk1
         ymin_ax0=min(Ne_short_pk2[:,1])-10
-        ymax_ax0=min(Ne_short_pk2[:,1])+config.y_range_baseline
+        ymax_ax0=min(Ne_short_pk2[:,1])+config.y_range_baseline_pk2
         ax1.set_ylim([ymin_ax1, ymax_ax1])
-        ax0.set_ylim([ymin_ax1, ymax_ax1])
+        ax0.set_ylim([ymin_ax0, ymax_ax0])
 
         # Setting x limits of axis
 
-        ax1_xmin=min(Ne_short_pk1[:,0])-config.x_range_baseline
-        ax1_xmax=max(Ne_short_pk1[:,0])+config.x_range_baseline
-        ax0_xmin=min(Ne_short_pk2[:,0])-config.x_range_baseline
-        ax0_xmax=max(Ne_short_pk2[:,0])+config.x_range_baseline
+        ax1_xmin=min(Ne_short_pk1[:,0])-config.x_range_baseline_pk1
+        ax1_xmax=max(Ne_short_pk1[:,0])+config.x_range_baseline_pk1
+        ax0_xmin=min(Ne_short_pk2[:,0])-config.x_range_baseline_pk2
+        ax0_xmax=max(Ne_short_pk2[:,0])+config.x_range_baseline_pk2
         ax0.set_xlim([ax0_xmin, ax0_xmax])
         ax1.set_xlim([ax1_xmin, ax1_xmax])
 
@@ -983,12 +1135,12 @@ Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
 
 
         ax1.plot([Ne_center_1, Ne_center_1], [ymin_ax1, ymax_ax1], ':k', label='Peak')
-        ax0.plot([Ne_center_2, Ne_center_2], [ymin_ax1, ymax_ax1], ':k', label='Peak')
+        ax0.plot([Ne_center_2, Ne_center_2], [ymin_ax0, ymax_ax0], ':k', label='Peak')
 
-        ax1.set_title('Peak1: 1117 background fitting')
+        ax1.set_title('%.0f' %Ne_center_1+': background fitting')
         ax1.set_xlabel('Wavenumber')
         ax1.set_ylabel('Intensity')
-        ax0.set_title('Peak2: 1447 background fitting')
+        ax0.set_title('%.0f' %Ne_center_2+ ': background fitting')
         ax0.set_xlabel('Wavenumber')
         ax0.set_ylabel('Intensity')
         ax1cop.set_xlabel('Offset from Pk estimate')
@@ -1080,11 +1232,14 @@ Ne_center_1=1117.1, Ne_center_2=1147, peaks_1=2,
                           'pk2_sigma': sigma_pk2,
                           'pk2_gamma': gamma_pk2,
                           'error_pk2': error_pk2,
+                          'Peak2_Prop_Lor': Peak2_Prop_Lor,
                           'pk1_peak_cent':cent_pk1,
                           'pk1_amplitude': Area_pk1,
                           'pk1_sigma': sigma_pk1,
                           'pk1_gamma': gamma_pk1,
                           'error_pk1': error_pk1,
+                          'Peak1_Prop_Lor': Peak1_Prop_Lor,
+
                          'deltaNe': DeltaNe,
                          'Ne_Corr': Ne_Corr,
                          'Ne_Corr_min':Ne_Corr_min,
@@ -1147,47 +1302,65 @@ def plot_Ne_corrections(df=None, x_axis=None, x_label='index', marker='o', mec='
     return fig
 
 ## Looping Ne lines
-def loop_Ne_lines(*, files, path, filetype,
-                  config, peaks_1,  Ne_center_1,
-                  Ne_center_2, DeltaNe_ideal, prefix=None,
-                  plot_figure=True, save_clipboard=True, single_acq=True):
+def loop_Ne_lines(*, files, spectra_path, filetype,
+        config, config_ID_peaks, df_fit_params=None, prefix=False, print_df=False,
+                  plot_figure=True, single_acq=False):
 
     df = pd.DataFrame([])
+    # This is for repeated acquisition of Ne lines
     if single_acq is True:
+
         for i in tqdm(range(0, np.shape(files)[1]-2)):
             Ne=np.column_stack((files[:, 0], files[:, i+1]))
             filename=str(i)
 
-            data=fit_Ne_lines(
-            config=config, peaks_1=peaks_1,
-            Ne=Ne, filename=filename, path=path, prefix=prefix,
-            Ne_center_1=Ne_center_1, Ne_center_2=Ne_center_2,
-            DeltaNe_ideal=DeltaNe_ideal, plot_figure=plot_figure,
-            save_clipboard=save_clipboard)
+            Ne, df_fit_params=identify_Ne_lines(Ne_array=Ne,
+            config=config_ID_peaks, print_df=False, plot_figure=False)
+
+
+            data=fit_Ne_lines(Ne=Ne, path=spectra_path,
+            config=config, prefix=prefix,
+            Ne_center_1=df_fit_params['Peak1_cent'].iloc[0],
+            Ne_center_2=df_fit_params['Peak2_cent'].iloc[0],
+            Ne_prom_1=df_fit_params['Peak1_prom'].iloc[0],
+            Ne_prom_2=df_fit_params['Peak2_prom'].iloc[0],
+            plot_figure=plot_figure)
             df = pd.concat([df, data], axis=0)
 
-
-    if single_acq is False:
+    else:
         for i in tqdm(range(0, len(files))):
             filename=files[i]
-            Ne=get_data(path=path, filename=filename, filetype=filetype)
-            data=fit_Ne_lines(
-            config=config, peaks_1=peaks_1,
-            Ne=Ne, filename=filename, path=path, prefix=prefix,
-            Ne_center_1=Ne_center_1, Ne_center_2=Ne_center_2,
-            DeltaNe_ideal=DeltaNe_ideal, plot_figure=plot_figure,
-            save_clipboard=save_clipboard)
+
+            Ne, df_fit_params=identify_Ne_lines(path=spectra_path,
+            filename=filename, filetype=filetype,
+            config=config_ID_peaks, print_df=False, plot_figure=False)
+
+            data=fit_Ne_lines(Ne=Ne, filename=filename,
+            path=spectra_path, prefix=prefix,
+            config=config,
+            Ne_center_1=df_fit_params['Peak1_cent'].iloc[0],
+            Ne_center_2=df_fit_params['Peak2_cent'].iloc[0],
+            Ne_prom_1=df_fit_params['Peak1_prom'].iloc[0],
+            Ne_prom_2=df_fit_params['Peak2_prom'].iloc[0],
+            plot_figure=plot_figure)
             df = pd.concat([df, data], axis=0)
-        #print('working on ' + str(files[i]))
 
 
 
 
 
+#print('working on ' + str(files[i]))
 
 
     df2=df.reset_index(drop=True)
 
+    # Now lets reorder some columns
+
+    cols_to_move = ['filename', 'Ne_Corr', 'deltaNe', 'pk2_peak_cent', 'pk1_peak_cent', 'pk2_amplitude', 'pk1_amplitude', 'residual_pk2', 'residual_pk1']
+    df2 = df2[cols_to_move + [
+                col for col in df2.columns if col not in cols_to_move]]
+
+
     return df2
 
 ## Regressing Ne lines against time
@@ -1228,8 +1401,8 @@ def reg_Ne_lines_time(df, fit='poly', N_poly=None, spline_fit=None):
 
     Py=Pf(Px)
 
-    fig, (ax1) = plt.subplots(1, 1, figsize=(5, 4))
-    ax1.plot(df['sec since midnight'], df['Ne_Corr'], 'ok')
+    fig, (ax1) = plt.subplots(1, 1, figsize=(6, 3))
+    ax1.plot(df['sec since midnight'], df['Ne_Corr'], 'xk')
     ax1.plot(Px, Py, '-r')
     ax1.set_xlabel('Seconds since midnight')
     ax1.set_ylabel('Ne Correction Factor')
@@ -1237,10 +1410,15 @@ def reg_Ne_lines_time(df, fit='poly', N_poly=None, spline_fit=None):
     ax1.ticklabel_format(useOffset=False)
 
 
-    return Pf
+    return Pf, fig
 
 
 def filter_Ne_Line_neighbours(Corr_factor, number_av=6, offset=0.00005):
+    """ This function discards Ne lines with a correction factor more than 
+    offset away from the median value of the N points (number_av) either side of it.
+    
+
+    """
     Corr_factor_Filt=np.empty(len(Corr_factor), dtype=float)
     median_loop=np.empty(len(Corr_factor), dtype=float)
 
@@ -1254,6 +1432,9 @@ def filter_Ne_Line_neighbours(Corr_factor, number_av=6, offset=0.00005):
         else:
             Corr_factor_Filt[i]=Corr_factor[i]
     ds=pd.Series(Corr_factor_Filt)
+
+
+
     return ds
 
 
